@@ -15,8 +15,8 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA
- * 02111-1307, USA.
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
+ * 02110-1301, USA.
  */
 
 #include "config.h"
@@ -126,16 +126,6 @@ record_set_pid (UTMP *u,
 #endif
 }
 
-static void
-record_set_id (UTMP       *u,
-               const char *id)
-{
-#if defined(HAVE_UT_UT_ID)
-        strncpy (u->ut_id, id, sizeof (u->ut_id));
-        g_debug ("using ut_id %.*s", (int) sizeof (u->ut_id), u->ut_id);
-#endif
-}
-
 static void
 record_set_host (UTMP       *u,
                  const char *x11_display_name,
@@ -161,11 +151,10 @@ record_set_host (UTMP       *u,
         if (hostname != NULL) {
                 strncpy (u->ut_host, hostname, sizeof (u->ut_host));
                 g_debug ("using ut_host %.*s", (int) sizeof (u->ut_host), u->ut_host);
-                g_free (hostname);
-
 #ifdef HAVE_UT_UT_SYSLEN
                 u->ut_syslen = MIN (strlen (hostname), sizeof (u->ut_host));
 #endif
+                g_free (hostname);
         }
 #endif
 }
@@ -185,8 +174,7 @@ record_set_line (UTMP       *u,
                 strncpy (u->ut_line,
                          display_device + strlen ("/dev/"),
                          sizeof (u->ut_line));
-        } else if (x11_display_name != NULL
-                   && g_str_has_prefix (x11_display_name, ":")) {
+        } else if (x11_display_name != NULL) {
                 strncpy (u->ut_line,
                          x11_display_name,
                          sizeof (u->ut_line));
@@ -203,7 +191,9 @@ gdm_session_record_login (GPid                  session_pid,
                           const char           *display_device)
 {
         UTMP        session_record = { 0 };
-        UTMP       *u;
+
+        if (x11_display_name == NULL)
+                x11_display_name = display_device;
 
         record_set_username (&session_record, user_name);
 
@@ -216,9 +206,6 @@ gdm_session_record_login (GPid                  session_pid,
 
         record_set_timestamp (&session_record);
         record_set_pid (&session_record, session_pid);
-
-        /* Set ut_id to the $DISPLAY value */
-        record_set_id (&session_record, x11_display_name);
         record_set_host (&session_record, x11_display_name, host_name);
         record_set_line (&session_record, display_device, x11_display_name);
 
@@ -236,31 +223,10 @@ gdm_session_record_login (GPid                  session_pid,
 #endif
 #endif
 
-        /*
-         * Handle utmp
-         * Update if entry already exists
-         */
+        /* Handle utmp */
 #if defined(HAVE_GETUTXENT)
-        setutxent ();
-
-        while ((u = getutxent ()) != NULL) {
-                if (u->ut_type == USER_PROCESS &&
-                    (session_record.ut_line != NULL &&
-                     (strncmp (u->ut_line, session_record.ut_line,
-                               sizeof (u->ut_line)) == 0 ||
-                      u->ut_pid == session_record.ut_pid))) {
-                        g_debug ("Updating existing utmp record");
-                        pututxline (&session_record);
-                        break;
-                }
-        }
-        endutxent ();
-
-        /* Add new entry if update did not work */
-        if (u == NULL) {
-                g_debug ("Adding new utmp record");
-                pututxline (&session_record);
-        }
+        g_debug ("Adding or updating utmp record for login");
+        pututxline (&session_record);
 #elif defined(HAVE_LOGIN)
 	login (&session_record);
 #endif
@@ -274,7 +240,9 @@ gdm_session_record_logout (GPid                  session_pid,
                            const char           *display_device)
 {
         UTMP        session_record = { 0 };
-        UTMP       *u;
+
+        if (x11_display_name == NULL)
+                x11_display_name = display_device;
 
         g_debug ("Writing logout record");
 
@@ -285,12 +253,9 @@ gdm_session_record_logout (GPid                  session_pid,
 
         record_set_timestamp (&session_record);
         record_set_pid (&session_record, session_pid);
-        /* Set ut_id to the $DISPLAY value */
-        record_set_id (&session_record, x11_display_name);
         record_set_host (&session_record, x11_display_name, host_name);
         record_set_line (&session_record, display_device, x11_display_name);
 
-
         /* Handle wtmp */
         g_debug ("Writing wtmp logout record to " GDM_NEW_SESSION_RECORDS_FILE);
 #if defined(HAVE_UPDWTMPX)
@@ -303,36 +268,8 @@ gdm_session_record_logout (GPid                  session_pid,
 
         /* Handle utmp */
 #if defined(HAVE_GETUTXENT)
-        setutxent ();
-
-        while ((u = getutxent ()) != NULL &&
-               (u = getutxid (&session_record)) != NULL) {
-
-                g_debug ("Removing utmp record");
-                if (u->ut_pid == session_pid &&
-                    u->ut_type == DEAD_PROCESS) {
-                        /* Already done */
-                        break;
-                }
-
-                u->ut_type = DEAD_PROCESS;
-#if defined(HAVE_UT_UT_TV)
-                u->ut_tv.tv_sec  = session_record.ut_tv.tv_sec;
-                u->ut_tv.tv_usec = session_record.ut_tv.tv_usec;
-#elif defined(HAVE_UT_UT_TIME)
-                u->ut_time = session_record.ut_time;
-#endif
-#ifdef HAVE_UT_UT_EXIT_E_TERMINATION
-                u->ut_exit.e_termination = 0;
-                u->ut_exit.e_exit = 0;
-#endif
-
-                pututxline (u);
-
-                break;
-        }
-
-        endutxent ();
+        g_debug ("Adding or updating utmp record for logout");
+        pututxline (&session_record);
 #elif defined(HAVE_LOGOUT)
         logout (session_record.ut_line);
 #endif
@@ -347,6 +284,9 @@ gdm_session_record_failed (GPid                  session_pid,
 {
         UTMP        session_record = { 0 };
 
+        if (x11_display_name == NULL)
+                x11_display_name = display_device;
+
         record_set_username (&session_record, user_name);
 
         g_debug ("Writing failed session attempt record");
@@ -358,14 +298,14 @@ gdm_session_record_failed (GPid                  session_pid,
 
         record_set_timestamp (&session_record);
         record_set_pid (&session_record, session_pid);
-        /* Set ut_id to the $DISPLAY value */
-        record_set_id (&session_record, x11_display_name);
         record_set_host (&session_record, x11_display_name, host_name);
         record_set_line (&session_record, display_device, x11_display_name);
 
+#if defined(HAVE_UPDWTMPX) || defined(HAVE_UPDWTMP)
         /* Handle btmp */
         g_debug ("Writing btmp failed session attempt record to "
                  GDM_BAD_SESSION_RECORDS_FILE);
+#endif
 
 #if defined(HAVE_UPDWTMPX)
         updwtmpx (GDM_BAD_SESSION_RECORDS_FILE, &session_record);
